###[DEF]###
[name		=Torsteuerung	]

[e#1		=Signal 0/1		]
[e#2		=Status 0/1		]
[e#3		=Sperren 0/1	]
[e#4		=Laufzeit: Öffnen (s) #init=20		]
[e#5		=Laufzeit: Schließen (s) #init=20	]

[a#1		=Trigger		]
[a#2		=Status			]

[v#1		=				]	Richtung
[v#2		=0				]	Timer
###[/DEF]###


###[HELP]###
Dieser Baustein kann z.B. zur Steuerung eines einfachen Garagentors verwendet werden. Der Torantrieb muss dabei über einen(!) Steuereingang verfügen, der das Tor nach dem Schema "Auf-Stop-Zu-Stop" verfahren lässt. 

Getriggert wird der Baustein mit einem neuen Telegramm (0 oder 1) an E1. An E2 muss der aktuelle Torstatus anliegen (z.B. Reed-Kontakt), wobei der Wert 0 für "geschlossen" und der Wert 1 für "offen" (bzw. nicht geschlossen) steht.

Die (Mindest-)Laufzeiten für das Öffnen und Schließen des Tores sind an E4 bzw. E5 anzugeben. Während dieser Laufzeiten wird A2 (Status) entsprechend gesetzt. Nach Ablauf der Laufzeiten wird A2 im Normalfall auf den Status "offen" (1) oder "geschlossen" (0) gesetzt, sofern die Statusinformation (Reed-Kontakt) an E2 einen korrespondierenden Wert erhalten hat. Andernfalls wird A2 auf den Status "unbekannter Zustand" (4) gesetzt.

Während des Schließens führt ein neues Telegram =1 an E1 zum sofortigen Abbruch und A1 wird auf 1 gesetzt (der Torantrieb interpretiert dies i.d.R. als "Stop"). A2 wird in diesem Fall auf den Status "unbekannter Zustand" (4) gesetzt.

Mit Hilfe von E3 kann kann das Schließen des Tores ggf. verhindert werden (z.B. Lichtschranke). Ist E3=1 kann das Tor nicht geschlossen werden (wohl aber geöffnet). 

<b>Hinweis:</b>
Ein bereits <i>eingeleiteter</i> Schließvorgang wird durch E3 nicht(!) abgebrochen!

<b>Wichtig:</b>
A1 wird ausschließlich auf 1 gesetzt (niemals auf 0). In der Regel ist daher die Nachschaltung eines <link>Impuls/Trigger-LBS***lbs_16000110</link> erforderlich, dessen Ausgang schließlich das KO des Torantriebs nach dem Schema "Auf-Stop-Zu-Stop" triggert.

<b>Achtung:</b>
Prinzipbedingt kann dieser Baustein nicht ermitteln, in welchem Zustand sich der Torantrieb <i>tatsächlich</i> befindet. Daher wird auf Grundlage von E2 (Reed-Kontakt) und den Laufzeiten versucht, den aktuellen (logischen) Zustand zu ermitteln. Sofern der Torantrieb zusätzlich auf anderem Wege beeinflusst wird (z.B. Funksender oder Not-Stopp durch ein Hindernis), kann dies zu unerwarteten Ergebnissen führen.


<h2>Ein- und Ausgänge</h2>
E1: 1=Öffnen, 0=Schließen
E2: i.d.R. Reed-Kontakt: 0=Tor ist geschlossen, 1=Tor ist offen oder läuft gerade
E3: Sperren: &ne;0 = Schließen verhindern (z.B. Lichtschranke), 0 = Schließen erlauben
E4: Laufzeit in Sekunden für den Öffnungsvorgang (muss mindestens(!) der tatsächlichen Laufzeit des Tores entsprechen)
E5: Laufzeit in Sekunden für den Schließvorgang (muss mindestens(!) der tatsächlichen Laufzeit des Tores entsprechen)

A1: 1=Tor Öffnen oder Schließen bzw. stoppen (Triggerausgang), A1 wird NIEMALS 0!
A2: 0=Tor ist geschlossen, 1=Tor ist geöffnet (bzw. nicht geschlossen), 2=Schließvorgang, 3=Öffnungsvorgang, 4=unbekannter Zustand
###[/HELP]###


###[LBS]###
<?
function LB_LBSID($id) {
	if ($E=logic_getInputs($id)) {

		if (logic_getState($id)==0) {

			if ($E[1]['refresh']==1) {

				if (!($E[4]['value']>=1)) {$E[4]['value']=1;}
				if (!($E[5]['value']>=1)) {$E[5]['value']=1;}

				//öffnen
				if ($E[1]['value']==1) { 

					if ($E[2]['value']==0) {		//Tor geschlossen?
						logic_setVar($id,1,1);		//öffnet
						logic_setOutput($id,1,1); 	//Tor triggern
						logic_setOutput($id,2,3);	//Status=3 (öffnet)
						logic_setVar($id,2,(getMicrotime()+$E[4]['value']));
						logic_setState($id,1,1000);
					}
					
				//schließen
				} else if ($E[1]['value']==0 && $E[3]['value']==0) {

					if ($E[2]['value']==1) {		//Tor offen? (oder läuft gerade!)
						logic_setVar($id,1,0);		//schließt
						logic_setOutput($id,1,1); 	//Tor triggern
						logic_setOutput($id,2,2);	//Status=2 (schließt)
						logic_setVar($id,2,(getMicrotime()+$E[5]['value']));
						logic_setState($id,1,1000);
					}

				}

			}
			
		} else {		
		
			if ($V=logic_getVars($id)) {
			
				//öffnet
				if ($V[1]==1) {
					
					if (getMicrotime()>=$V[2]) {
						if ($E[2]['value']==1) {		//Tor offen? (bzw. nicht geschlossen)
							logic_setOutput($id,2,1);	//Status=1 (offen)
						} else {
							logic_setOutput($id,2,4);	//Status=4 (FEHLER)
						}
						logic_setState($id,0);
					}
					
				//schließt
				} else {

					//Notfall-Trigger während des Schließens
					if ($E[1]['refresh']==1 && $E[1]['value']==1) {

						logic_setOutput($id,1,1); 	//Tor triggern
						logic_setOutput($id,2,4);	//Status=4 (FEHLER)
						logic_setState($id,0);		//Abbruch

					} else {

						if (getMicrotime()>=$V[2] || $E[2]['value']==0) {
							if ($E[2]['value']==0) {		//Tor geschlossen?
								logic_setOutput($id,2,0);	//Status=0 (geschlossen)
							} else {
								logic_setOutput($id,2,4);	//Status=4 (FEHLER)
							}
							logic_setState($id,0);
						}

					}

				}
			}
		}
	}
}
?>
###[/LBS]###


###[EXEC]###
<?
?>
###[/EXEC]###
